Documentation.addTranslations({
    "locale": "ta",
    "messages": {
        "%(filename)s &#8212; %(docstitle)s": "%(\u0b95\u0bcb\u0baa\u0bcd\u0baa\u0bc1 \u0baa\u0bc6\u0baf\u0bb0\u0bcd) \u0b8e\u0bb8\u0bcd &#8212; %(\u0b9f\u0bbe\u0b95\u0bcd\u0bb8\u0bcd\u0b9f\u0bbf\u0b9f\u0bbf\u0bb2\u0bcd) \u0b95\u0bb3\u0bcd",
        "&#169; %(copyright_prefix)s %(copyright)s.": "&#169; %(\u0baa\u0ba4\u0bbf\u0baa\u0bcd\u0baa\u0bc1\u0bb0\u0bbf\u0bae\u0bc8_\u0baa\u0bbf\u0bb0\u0bc6\u0b83\u0baa\u0bbf\u0b95\u0bcd\u0bb8\u0bcd) \u0b8e\u0bb8\u0bcd %(\u0baa\u0ba4\u0bbf\u0baa\u0bcd\u0baa\u0bc1\u0bb0\u0bbf\u0bae\u0bc8) \u0b95\u0bb3\u0bcd.",
        ", in ": ", \u0b87\u0bb2\u0bcd",
        "About these documents": "\u0b87\u0ba8\u0bcd\u0ba4 \u0b86\u0bb5\u0ba3\u0b99\u0bcd\u0b95\u0bb3\u0bc8\u0baa\u0bcd \u0baa\u0bb1\u0bcd\u0bb1\u0bbf",
        "Automatically generated list of changes in version %(version)s": "\u0baa\u0ba4\u0bbf\u0baa\u0bcd\u0baa\u0bc1 %(\u0baa\u0ba4\u0bbf\u0baa\u0bcd\u0baa\u0bc1) S \u0b87\u0bb2\u0bcd \u0bae\u0bbe\u0bb1\u0bcd\u0bb1\u0b99\u0bcd\u0b95\u0bb3\u0bbf\u0ba9\u0bcd \u0baa\u0b9f\u0bcd\u0b9f\u0bbf\u0baf\u0bb2\u0bcd \u0ba4\u0bbe\u0ba9\u0bbe\u0b95 \u0b89\u0bb0\u0bc1\u0bb5\u0bbe\u0b95\u0bcd\u0b95\u0baa\u0bcd\u0baa\u0b9f\u0bc1\u0b95\u0bbf\u0bb1\u0ba4\u0bc1",
        "C API changes": "\u0b9a\u0bbf \u0b8f\u0baa\u0bbf\u0b90 \u0bae\u0bbe\u0bb1\u0bcd\u0bb1\u0b99\u0bcd\u0b95\u0bb3\u0bcd",
        "Changes in Version %(version)s &#8212; %(docstitle)s": "\u0baa\u0ba4\u0bbf\u0baa\u0bcd\u0baa\u0bc1 %(\u0baa\u0ba4\u0bbf\u0baa\u0bcd\u0baa\u0bc1) S &#8212; %(\u0b9f\u0bbe\u0b95\u0bcd\u0bb8\u0bcd\u0b9f\u0bbf\u0b9f\u0bbf\u0bb2\u0bcd) \u0b95\u0bb3\u0bcd",
        "Collapse sidebar": "\u0baa\u0b95\u0bcd\u0b95\u0baa\u0bcd\u0baa\u0b9f\u0bcd\u0b9f\u0bbf \u0b9a\u0bb0\u0bbf\u0bb5\u0bc1",
        "Complete Table of Contents": "\u0b89\u0bb3\u0bcd\u0bb3\u0b9f\u0b95\u0bcd\u0b95\u0b99\u0bcd\u0b95\u0bb3\u0bbf\u0ba9\u0bcd \u0bae\u0bc1\u0bb4\u0bc1\u0bae\u0bc8\u0baf\u0bbe\u0ba9 \u0b85\u0b9f\u0bcd\u0b9f\u0bb5\u0ba3\u0bc8",
        "Contents": "\u0b89\u0bb3\u0bcd\u0bb3\u0b9f\u0b95\u0bcd\u0b95\u0b99\u0bcd\u0b95\u0bb3\u0bcd",
        "Copyright": "\u0baa\u0ba4\u0bbf\u0baa\u0bcd\u0baa\u0bc1\u0bb0\u0bbf\u0bae\u0bc8",
        "Created using <a href=\"https://www.sphinx-doc.org/\">Sphinx</a> %(sphinx_version)s.": "<a href=\"https://www.sphinx-doc.org/\"> \u0bb8\u0bcd\u0baa\u0bbf\u0ba9\u0bcd\u0b95\u0bcd\u0bb8\u0bcd </\u0bae\u0bb1\u0bcd\u0bb1\u0bc1\u0bae\u0bcd> %(\u0bb8\u0bcd\u0baa\u0bbf\u0ba9\u0bcd\u0b95\u0bcd\u0bb8\u0bcd \u0baa\u0ba4\u0bbf\u0baa\u0bcd\u0baa\u0bc1\u0b95\u0bb3\u0bcd.",
        "Expand sidebar": "\u0baa\u0b95\u0bcd\u0b95\u0baa\u0bcd\u0baa\u0b9f\u0bcd\u0b9f\u0bbf\u0baf\u0bc8 \u0bb5\u0bbf\u0bb0\u0bbf\u0bb5\u0bbe\u0b95\u0bcd\u0b95\u0bc1\u0b99\u0bcd\u0b95\u0bb3\u0bcd",
        "Full index on one page": "\u0b92\u0bb0\u0bc1 \u0baa\u0b95\u0bcd\u0b95\u0ba4\u0bcd\u0ba4\u0bbf\u0bb2\u0bcd \u0bae\u0bc1\u0bb4\u0bc1 \u0b95\u0bc1\u0bb1\u0bbf\u0baf\u0bc0\u0b9f\u0bc1",
        "General Index": "\u0baa\u0bca\u0ba4\u0bc1 \u0b85\u0b9f\u0bcd\u0b9f\u0bb5\u0ba3\u0bc8",
        "Global Module Index": "\u0b89\u0bb2\u0b95\u0bb3\u0bbe\u0bb5\u0bbf\u0baf \u0ba4\u0bca\u0b95\u0bc1\u0ba4\u0bbf \u0b85\u0b9f\u0bcd\u0b9f\u0bb5\u0ba3\u0bc8",
        "Go": "\u0baa\u0bc7\u0bbe",
        "Hide Search Matches": "\u0ba4\u0bc7\u0b9f\u0bb2\u0bcd \u0baa\u0bcb\u0b9f\u0bcd\u0b9f\u0bbf\u0b95\u0bb3\u0bc8 \u0bae\u0bb1\u0bc8\u0b95\u0bcd\u0b95\u0bb5\u0bc1\u0bae\u0bcd",
        "Index": "\u0b95\u0bc1\u0bb1\u0bbf\u0baf\u0bc0\u0b9f\u0bcd\u0b9f\u0bc1",
        "Index &ndash; %(key)s": "\u0b95\u0bc1\u0bb1\u0bbf\u0baf\u0bc0\u0b9f\u0bcd\u0b9f\u0bc1 & ndash; %(\u0bb5\u0bbf\u0b9a\u0bc8\u0b95\u0bb3\u0bcd",
        "Index pages by letter": "\u0b95\u0b9f\u0bbf\u0ba4\u0bae\u0bcd \u0bae\u0bc2\u0bb2\u0bae\u0bcd \u0b95\u0bc1\u0bb1\u0bbf\u0baf\u0bc0\u0b9f\u0bcd\u0b9f\u0bc1 \u0baa\u0b95\u0bcd\u0b95\u0b99\u0bcd\u0b95\u0bb3\u0bcd",
        "Indices and tables:": "\u0b95\u0bc1\u0bb1\u0bbf\u0baf\u0bc0\u0b9f\u0bc1\u0b95\u0bb3\u0bcd \u0bae\u0bb1\u0bcd\u0bb1\u0bc1\u0bae\u0bcd \u0b85\u0b9f\u0bcd\u0b9f\u0bb5\u0ba3\u0bc8\u0b95\u0bb3\u0bcd:",
        "Last updated on %(last_updated)s.": "\u0b95\u0b9f\u0bc8\u0b9a\u0bbf\u0baf\u0bbe\u0b95 %(last_updated) s \u0b87\u0bb2\u0bcd \u0baa\u0bc1\u0ba4\u0bc1\u0baa\u0bcd\u0baa\u0bbf\u0b95\u0bcd\u0b95\u0baa\u0bcd\u0baa\u0b9f\u0bcd\u0b9f\u0ba4\u0bc1.",
        "Library changes": "\u0ba8\u0bc2\u0bb2\u0b95 \u0bae\u0bbe\u0bb1\u0bcd\u0bb1\u0b99\u0bcd\u0b95\u0bb3\u0bcd",
        "Navigation": "\u0bb5\u0bb4\u0bbf\u0b9a\u0bc6\u0bb2\u0bc1\u0ba4\u0bcd\u0ba4\u0bb2\u0bcd",
        "Next topic": "\u0b85\u0b9f\u0bc1\u0ba4\u0bcd\u0ba4 \u0ba4\u0bb2\u0bc8\u0baa\u0bcd\u0baa\u0bc1",
        "Other changes": "\u0baa\u0bbf\u0bb1 \u0bae\u0bbe\u0bb1\u0bcd\u0bb1\u0b99\u0bcd\u0b95\u0bb3\u0bcd",
        "Overview": "\u0b95\u0ba3\u0bcd\u0ba3\u0bcb\u0b9f\u0bcd\u0b9f\u0bae\u0bcd",
        "Please activate JavaScript to enable the search\n    functionality.": "\u0ba4\u0bc7\u0b9f\u0bb2\u0bc8 \u0b87\u0baf\u0b95\u0bcd\u0b95 \u0b9c\u0bbe\u0bb5\u0bbe\u0bb8\u0bcd\u0b95\u0bbf\u0bb0\u0bbf\u0baa\u0bcd\u0b9f\u0bc8 \u0b9a\u0bc6\u0baf\u0bb2\u0bcd\u0baa\u0b9f\u0bc1\u0ba4\u0bcd\u0ba4\u0bb5\u0bc1\u0bae\u0bcd \n     \u0b9a\u0bc6\u0baf\u0bb2\u0bcd\u0baa\u0bbe\u0b9f\u0bc1.",
        "Preparing search...": "\u0ba4\u0bc7\u0b9f\u0bb2\u0bc8\u0ba4\u0bcd \u0ba4\u0baf\u0bbe\u0bb0\u0bbf\u0ba4\u0bcd\u0ba4\u0bb2\u0bcd ...",
        "Previous topic": "\u0bae\u0bc1\u0ba8\u0bcd\u0ba4\u0bc8\u0baf \u0ba4\u0bb2\u0bc8\u0baa\u0bcd\u0baa\u0bc1",
        "Quick search": "\u0bb5\u0bbf\u0bb0\u0bc8\u0bb5\u0bc1 \u0ba4\u0bc7\u0b9f\u0bb2\u0bcd",
        "Search": "\u0ba4\u0bc7\u0b9f\u0bb2\u0bcd",
        "Search Page": "\u0ba4\u0bc7\u0b9f\u0bb2\u0bcd \u0baa\u0b95\u0bcd\u0b95\u0bae\u0bcd",
        "Search Results": "\u0ba4\u0bc7\u0b9f\u0bb2\u0bcd \u0bae\u0bc1\u0b9f\u0bbf\u0bb5\u0bc1\u0b95\u0bb3\u0bcd",
        "Search finished, found ${resultCount} page(s) matching the search query.": "\u0ba4\u0bc7\u0b9f\u0bb2\u0bcd \u0bae\u0bc1\u0b9f\u0bbf\u0ba8\u0bcd\u0ba4\u0ba4\u0bc1, \u0ba4\u0bc7\u0b9f\u0bb2\u0bcd \u0bb5\u0bbf\u0ba9\u0bb5\u0bb2\u0bc1\u0b9f\u0ba9\u0bcd \u0baa\u0bca\u0bb0\u0bc1\u0ba8\u0bcd\u0ba4\u0b95\u0bcd\u0b95\u0bc2\u0b9f\u0bbf\u0baf $ {resultCount} \u0baa\u0b95\u0bcd\u0b95\u0bae\u0bcd (\u0b95\u0bb3\u0bcd) \u0b95\u0bbf\u0b9f\u0bc8\u0ba4\u0bcd\u0ba4\u0ba4\u0bc1.",
        "Search within %(docstitle)s": "%(\u0b9f\u0bbe\u0b95\u0bcd\u0bb8\u0bcd\u0b9f\u0bbf\u0b9f\u0bbf\u0bb2\u0bcd) \u0b95\u0bb3\u0bcd \u0ba4\u0bc7\u0b9f\u0bc1\u0b99\u0bcd\u0b95\u0bb3\u0bcd",
        "Searching": "\u0ba4\u0bc7\u0b9f\u0bbf",
        "Searching for multiple words only shows matches that contain\n    all words.": "\u0baa\u0bb2 \u0b9a\u0bca\u0bb1\u0bcd\u0b95\u0bb3\u0bc8\u0ba4\u0bcd \u0ba4\u0bc7\u0b9f\u0bc1\u0bb5\u0ba4\u0bc1 \u0b95\u0bbe\u0ba3\u0bcd\u0baa\u0bbf\u0b95\u0bcd\u0b95\u0bc1\u0bae\u0bcd \u0baa\u0bcb\u0b9f\u0bcd\u0b9f\u0bbf\u0b95\u0bb3\u0bc8 \u0bae\u0b9f\u0bcd\u0b9f\u0bc1\u0bae\u0bc7 \u0b95\u0bbe\u0b9f\u0bcd\u0b9f\u0bc1\u0b95\u0bbf\u0bb1\u0ba4\u0bc1 \n     \u0b85\u0ba9\u0bc8\u0ba4\u0bcd\u0ba4\u0bc1 \u0bb5\u0bbe\u0bb0\u0bcd\u0ba4\u0bcd\u0ba4\u0bc8\u0b95\u0bb3\u0bc1\u0bae\u0bcd.",
        "Show Source": "\u0bae\u0bc2\u0bb2\u0ba4\u0bcd\u0ba4\u0bc8\u0b95\u0bcd \u0b95\u0bbe\u0b9f\u0bcd\u0b9f\u0bc1",
        "Table of Contents": "\u0b89\u0bb3\u0bcd\u0bb3\u0b9f\u0b95\u0bcd\u0b95 \u0b85\u0b9f\u0bcd\u0b9f\u0bb5\u0ba3\u0bc8",
        "This Page": "\u0b87\u0ba8\u0bcd\u0ba4 \u0baa\u0b95\u0bcd\u0b95\u0bae\u0bcd",
        "Welcome! This is": "\u0bb5\u0bb0\u0bb5\u0bc7\u0bb1\u0bcd\u0baa\u0bc1! \u0b87\u0ba4\u0bc1",
        "Your search did not match any documents. Please make sure that all words are spelled correctly and that you've selected enough categories.": "\u0b89\u0b99\u0bcd\u0b95\u0bb3\u0bcd \u0ba4\u0bc7\u0b9f\u0bb2\u0bcd \u0b8e\u0ba8\u0bcd\u0ba4 \u0b86\u0bb5\u0ba3\u0b99\u0bcd\u0b95\u0bb3\u0bc1\u0b95\u0bcd\u0b95\u0bc1\u0bae\u0bcd \u0baa\u0bca\u0bb0\u0bc1\u0ba8\u0bcd\u0ba4\u0bb5\u0bbf\u0bb2\u0bcd\u0bb2\u0bc8. \u0b8e\u0bb2\u0bcd\u0bb2\u0bbe \u0b9a\u0bca\u0bb1\u0bcd\u0b95\u0bb3\u0bc1\u0bae\u0bcd \u0b9a\u0bb0\u0bbf\u0baf\u0bbe\u0b95 \u0b89\u0b9a\u0bcd\u0b9a\u0bb0\u0bbf\u0b95\u0bcd\u0b95\u0baa\u0bcd\u0baa\u0b9f\u0bcd\u0b9f\u0bc1\u0bb3\u0bcd\u0bb3\u0ba9 \u0b8e\u0ba9\u0bcd\u0baa\u0ba4\u0bc8\u0baf\u0bc1\u0bae\u0bcd, \u0ba8\u0bc0\u0b99\u0bcd\u0b95\u0bb3\u0bcd \u0baa\u0bcb\u0ba4\u0bc1\u0bae\u0bbe\u0ba9 \u0bb5\u0b95\u0bc8\u0b95\u0bb3\u0bc8\u0ba4\u0bcd \u0ba4\u0bc7\u0bb0\u0bcd\u0ba8\u0bcd\u0ba4\u0bc6\u0b9f\u0bc1\u0ba4\u0bcd\u0ba4\u0bc1\u0bb3\u0bcd\u0bb3\u0bc0\u0bb0\u0bcd\u0b95\u0bb3\u0bcd \u0b8e\u0ba9\u0bcd\u0baa\u0ba4\u0bc8\u0baf\u0bc1\u0bae\u0bcd \u0b89\u0bb1\u0bc1\u0ba4\u0bbf\u0baa\u0bcd\u0baa\u0b9f\u0bc1\u0ba4\u0bcd\u0ba4\u0bb5\u0bc1\u0bae\u0bcd.",
        "all functions, classes, terms": "\u0b85\u0ba9\u0bc8\u0ba4\u0bcd\u0ba4\u0bc1 \u0b9a\u0bc6\u0baf\u0bb2\u0bcd\u0baa\u0bbe\u0b9f\u0bc1\u0b95\u0bb3\u0bcd, \u0bb5\u0b95\u0bc1\u0baa\u0bcd\u0baa\u0bc1\u0b95\u0bb3\u0bcd, \u0bb5\u0bbf\u0ba4\u0bbf\u0bae\u0bc1\u0bb1\u0bc8\u0b95\u0bb3\u0bcd",
        "can be huge": "\u0bae\u0bbf\u0b95\u0baa\u0bcd\u0baa\u0bc6\u0bb0\u0bbf\u0baf\u0ba4\u0bbe\u0b95 \u0b87\u0bb0\u0bc1\u0b95\u0bcd\u0b95 \u0bae\u0bc1\u0b9f\u0bbf\u0baf\u0bc1\u0bae\u0bcd",
        "last updated": "\u0b95\u0b9f\u0bc8\u0b9a\u0bbf\u0baf\u0bbe\u0b95 \u0baa\u0bc1\u0ba4\u0bc1\u0baa\u0bcd\u0baa\u0bbf\u0b95\u0bcd\u0b95\u0baa\u0bcd\u0baa\u0b9f\u0bcd\u0b9f\u0ba4\u0bc1",
        "lists all sections and subsections": "\u0b85\u0ba9\u0bc8\u0ba4\u0bcd\u0ba4\u0bc1 \u0baa\u0bbf\u0bb0\u0bbf\u0bb5\u0bc1\u0b95\u0bb3\u0bc8\u0baf\u0bc1\u0bae\u0bcd \u0ba4\u0bc1\u0ba3\u0bc8\u0baa\u0bcd\u0baa\u0bbf\u0bb0\u0bbf\u0bb5\u0bc1\u0b95\u0bb3\u0bc8\u0baf\u0bc1\u0bae\u0bcd \u0baa\u0b9f\u0bcd\u0b9f\u0bbf\u0baf\u0bb2\u0bbf\u0b9f\u0bc1\u0b95\u0bbf\u0bb1\u0ba4\u0bc1",
        "next chapter": "\u0b85\u0b9f\u0bc1\u0ba4\u0bcd\u0ba4 \u0b85\u0ba4\u0bcd\u0ba4\u0bbf\u0baf\u0bbe\u0baf\u0bae\u0bcd",
        "previous chapter": "\u0bae\u0bc1\u0ba8\u0bcd\u0ba4\u0bc8\u0baf \u0b85\u0ba4\u0bcd\u0ba4\u0bbf\u0baf\u0bbe\u0baf\u0bae\u0bcd",
        "quick access to all modules": "\u0b85\u0ba9\u0bc8\u0ba4\u0bcd\u0ba4\u0bc1 \u0ba4\u0bca\u0b95\u0bc1\u0ba4\u0bbf\u0b95\u0bb3\u0bc1\u0b95\u0bcd\u0b95\u0bc1\u0bae\u0bcd \u0bb5\u0bbf\u0bb0\u0bc8\u0bb5\u0bbe\u0ba9 \u0b85\u0ba3\u0bc1\u0b95\u0bb2\u0bcd",
        "search": "\u0ba4\u0bc7\u0b9f\u0bb2\u0bcd",
        "search this documentation": "\u0b87\u0ba8\u0bcd\u0ba4 \u0b86\u0bb5\u0ba3\u0ba4\u0bcd\u0ba4\u0bc8\u0ba4\u0bcd \u0ba4\u0bc7\u0b9f\u0bc1\u0b99\u0bcd\u0b95\u0bb3\u0bcd",
        "the documentation for": "\u0b86\u0bb5\u0ba3\u0b99\u0bcd\u0b95\u0bb3\u0bcd"
    },
    "plural_expr": "(n != 1)"
});